/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/*          EXIFPROBE - TIFF/JPEG/EXIF image file probe               */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Copyright (C) 2002 by Duane H. Hesser. All rights reserved.        */
/*                                                                    */
/* See the file LICENSE.EXIFPROBE for terms of use.                   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#ifndef lint
static char *ModuleId = "@(#) $Id: maker_fujifilm.c,v 1.10 2005/01/27 20:09:27 alex Exp $";
#endif

/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Fujitsu camera maker-specific routines (Fujifilm)                  */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Most of the information coded here is due to TsuruZoh Tachibanaya  */
/* at:                                                                */
/* http://www.ba.wakwak.com/~tsuruzoh/Computer/Digicams/exif-e.html   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#include <stdio.h>
#include <string.h>
#include "defs.h"
#include "extern.h"
#include "misc.h"
#include "maker.h"

/* Find the identifying number assigned to known Fujifilm camera      */
/* models. This number is used to dispatch print and interpret        */
/* routines approopriate to the current image.                        */

int
fujifilm_model_number(char *model,char *software)
{
    int number = GENERIC;

    if(model)
    {
        switch(*model)
        {
            case 'D':
                /* This could be cleverer                         */
                if(strncmp(model,"Digital Camera FinePix F601",27) == 0)
                    number = FUJI_F601;
                else if(strcmp(model,"DX-10") == 0)
                    number = FUJI_DX10;
                break;
            case 'F':
                if(strcmp(model,"FinePix40i") == 0)
                    number = FUJI_40i;
                else if(strncmp(model,"FinePix4900",10) == 0)
                    number = FUJI_4900;
                else if(strncmp(model,"FinePixS1Pro",11) == 0)
                    number = FUJI_S1PRO;
                else if(strncmp(model,"FinePixS2Pro",11) == 0)
                    number = FUJI_S2PRO;
                break;
            case 'M':
                if(strncmp(model,"MX-1700",7) == 0)
                    number = FUJI_MX1700;
                break;
            default:
                number = GENERIC;
                break;
        }
    }
    return(number);
}


/* Dispatch a print routine for direct values in Fujifilm cameras,    */
/* based upon model                                                   */

void
print_fujifilm_makervalue(struct ifd_entry *entry_ptr,int make,int model,
                    char *prefix)
{

    if(entry_ptr && (PRINT_VALUE))
    {
        switch(model)
        {
            case FUJI_F601:
            case FUJI_DX10:
            case FUJI_40i:
            case FUJI_4900:
            case FUJI_MX1700:
            case FUJI_S1PRO:
            case FUJI_S2PRO:
            case GENERIC:
            default:
                print_fujifilm1_makervalue(entry_ptr,make,model,prefix);
                fujifilm1_interpret_value(entry_ptr);
                break;
        }
    }
}

/* Model-specific print routine for Fujifilm cameras. This routine is */
/* responsible for picking off any direct entry tags which are        */
/* peculiar and will not be handled properly by print_value()         */
/* (usually UNDEFINED values which fit in the 4-byte entry value). If */
/* there are no such entries, this function simply calls              */
/* print_value().                                                     */

void
print_fujifilm1_makervalue(struct ifd_entry *entry_ptr,int make,int model,
                    char *prefix)
{
    if(entry_ptr && (PRINT_VALUE))
    {
        switch(entry_ptr->tag) 
        {
            default:
                print_value(entry_ptr,prefix);
                break;
        }
    }
}


/* Dispatch a routine to decode and print offset values for Fujifilm  */
/* cameras.                                                           */

void
print_fujifilm_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{

    if(entry_ptr && (PRINT_VALUE))
    {
        switch(model)
        {
            /* currently only one routine available                       */
            default:
                fujifilm1_offset_makervalue(inptr,byteorder,entry_ptr,
                                            offset_ref,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
        }
    }
}

/* Model-specific routine to print UNDEFINED values found at offsets  */
/* in Fujifilm makernotes.                                            */

void
fujifilm1_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{
    if(entry_ptr)
    {
        switch(entry_ptr->tag)
        {
            default:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                        offset_ref,prefix,indent,
                                        addrwidth,make,model,at_offset);
                break;
        }
    }
}


/* Fujifilm-specific tagnames for makernotes.                              */

char *
maker_fujifilm_tagname(unsigned short tag,int model)
{
    char *tagname = (char *)0;

    /* Check specific models first                                    */
    switch(model)
    {
        /* Add special-case models here                               */
        default:
            break;
    }

    /* If no model-specific tag is found, use a "generic" Fujifilm    */
    /* tag                                                            */
    if(tagname == NULL)
    {
        switch(tag)
        {
            case 0x0000: tagname = "Version"; break;
            case 0x1000: tagname = "Quality"; break;
            case 0x1001: tagname = "Sharpness"; break;
            case 0x1002: tagname = "WhiteBalance"; break;
            case 0x1003: tagname = "Color"; break;
            case 0x1004: tagname = "Tone"; break;
            case 0x1010: tagname = "FlashMode"; break;
            case 0x1011: tagname = "FlashStrength"; break;
            case 0x1020: tagname = "Macro"; break;
            case 0x1021: tagname = "FocusMode"; break;
            case 0x1030: tagname = "SlowSync"; break;
            case 0x1031: tagname = "PictureMode"; break;
            case 0x1100: tagname = "ContTake-Bracket"; break;
            case 0x1300: tagname = "BlurWarning"; break;
            case 0x1301: tagname = "FocusWarning"; break;
            case 0x1302: tagname = "AEWarning"; break;
            default: break;
        }
    }
    return(tagname);
}


/* Report the "meaning" of tag values found at offsets in a Fujifilm  */
/* MakerNote IFD entry (not at an offset).                            */

void
fujifilm1_interpret_value(struct ifd_entry *entry_ptr)
{
    switch(entry_ptr->tag)
    {
        case 0x1001:    /* Sharpness */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Soft"); break;
                case 2: printf("Soft"); break;
                case 3: printf("Normal"); break;
                case 4: printf("Hard"); break;
                case 5: printf("Hard"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x1002:    /* White Balance */
            print_startvalue();
            /* I think we're looking at a bit mask here, but I have   */
            /* no way to check; this is the way Tachibanaya           */
            /* characterizes it.                                      */
            switch(entry_ptr->value)
            {
                case 0: printf("Auto"); break;
                case 256: printf("Daylight"); break;
                case 512: printf("Cloudy"); break;
                case 768:   /* ???? */ printf("DaylightColor-Fluorescent"); break;
                case 769:   /* ???? */ printf("DaywhiteColor-Fluorescent"); break;
                case 770:   /* ???? */ printf("White-Fluorescent"); break;
                case 1024: printf("Incandescent"); break;
                case 3840: printf("Custom"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x1003:    /* Color Saturation */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Standard"); break;
                case 256: printf("High"); break;
                case 512: printf("Low"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x1004:    /* Tone */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Standard"); break;
                case 256: printf("High"); break;
                case 512: printf("Low"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x1010:    /* Flash Mode */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Auto"); break;
                case 1: printf("On"); break;
                case 2: printf("Off"); break;
                case 3: printf("Redeye"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x1020:    /* Macro Mode */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Off"); break;
                case 1: printf("On"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x1021:    /* Focusing Mode */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Auto"); break;
                case 1: printf("Manual"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x1030:    /* Slow Sync */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Off"); break;
                case 1: printf("On"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x1031:    /* Picture Mode */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Auto"); break;
                case 1: printf("Portrait"); break;
                case 2: printf("Landscape"); break;
                case 4: printf("Sports"); break;
                case 5: printf("Night"); break;
                case 6: printf("Program Normal"); break;
                case 256: printf("Aperture Priority"); break;
                case 512: printf("Shutter Priority"); break;
                case 768: printf("Manual"); break;
                default: printred("undefined"); }
            print_endvalue();
            break;
        case 0x1100:    /* Continuous Take/Bracket */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Off"); break;
                case 1: printf("On"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x1300:    /* Blur Warning */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Ok"); break;
                case 1: printf("Blurred"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x1301:    /* Focus Warning */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Ok"); break;
                case 1: printf("Out of focus"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x1302:    /* AE Warning */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Ok"); break;
                case 1: printf("Overexposed"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        default:
            break;
    }
}
